/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions;

import org.apache.commons.collections4.IterableUtils;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.genotype.ProvidingGenotype;
import org.fortiss.af3.exploration.alg.dse.sysmodel.random.RandomExploration;
import org.opt4j.core.Genotype;
import org.opt4j.core.genotype.CompositeGenotype;
import org.opt4j.operators.mutate.MutateGenericImplementation;

import com.google.inject.Singleton;

/**
 * Implementation of the {@link MutateGenericImplementation} for exploration processes: Only a
 * single mutation operation is applied in contrast to applying the mutate operation to each sub
 * {@link Genotype}. The latter wastes computations since the genotypes are modified but overridden
 * in the decode phase to maintain consistency.
 * 
 * @author diewald
 */
@Singleton
public class MutateGenericProcessImplementation extends MutateGenericImplementation {

	/** References the random number generator instance. */
	private RandomExploration randExp = RandomExploration.getInstance();

	/** Executes the mutate operation . */
	@Override
	protected void mutateComposite(CompositeGenotype<?, ?> genotype, double p) {
		Genotype subGenotype = genotype;
		if(genotype instanceof ProvidingGenotype) {
			subGenotype = ((ProvidingGenotype)genotype).getModifiedGeneotype();
			int selectedGenotypeIdx = randExp.nextInt(genotype.size());
			// No NPE checked for the getter --> would indicate a framework problem.
			subGenotype = IterableUtils.get(genotype, selectedGenotypeIdx).getValue();
		}
		mutate(subGenotype, p);
	}
}
