/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.operator.selection;

import org.opt4j.core.Genotype;
import org.opt4j.core.optimizer.Operator;
import org.opt4j.operators.OperatorModule;

/**
 * Interface for adding (possibly multiple) operators to multiple {@link Genotype}s. The
 * {@link IOperatorSelector} is used to select one operator that shall be aplied for the given given
 * {@link Genotype}.
 * 
 * @author barner
 */
public interface IMultiSelectorOperatorModule<O extends Operator<?>> {
	/**
	 * Registers a pre-constructed mutation operator instance with an {@link OperatorModule} and a
	 * specific {@link Genotype} provided by the Opt4J framework. The operator will be applied only
	 * to the given {@link Genotype}.
	 */
	void addOperatorInstance(Class<? extends Genotype> genotypeType, O crossoverOperator);

	/**
	 * Registers a mutation operator type with an {@link OperatorModule} and a specific
	 * {@link Genotype} provided by the Opt4J framework. The operator will be applied only
	 * to the given {@link Genotype}.
	 */
	void addOperatorType(Class<? extends Genotype> genotypeType, Class<O> mutateOperator);

	/**
	 * Returns the {@link IOperatorSelector} used to manage the selection of operators registered
	 * with the instance of the class implementing this interface (used internally by
	 * {@link #addOperatorInstance(Class, Operator)}).
	 * 
	 * Implementations must ensure that the same {@link IOperatorSelector} instance is returned for
	 * all calls of {@link #addOperatorInstance(Class, Operator)}.
	 */
	IOperatorSelector getOperatorSelector(Class<? extends Genotype> genotypeType);
}
