/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.evaluator.objective;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.decode.Genotyped;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.evaluate.Evaluate;
import org.fortiss.af3.exploration.alg.dse.module.EvaluatorModule;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.moea.model.objectives.PartitionMinObjective;
import org.opt4j.core.DoubleValue;

import com.google.inject.Provides;

/**
 * Returns the sum of partitions present in a system to be explored.
 * 
 * @author diewald
 */
public class PartitionNumberEvaluator
		extends EvaluatorModule<Double, PartitionMinObjective, PartitionMappingEncoding> {

	/** Constructor. */
	public PartitionNumberEvaluator(Class<? extends PartitionMinObjective> explorationTargetType) {
		super(explorationTargetType);
	}

	/** Constructor. */
	public PartitionNumberEvaluator(PartitionMinObjective explorationTarget) {
		super(explorationTarget);
	}

	/**
	 * Evaluates the reliability of an {@link ITaskAdapter} given the current mapping and schedule.
	 */
	@Provides
	@Evaluate(PartitionNumberEvaluator.class)
	public DoubleValue evaluate(@Genotyped PartitionMappingEncoding pmEnc) {
		return new DoubleValue(Double.valueOf(pmEnc.getRequesters().size()));
	}
}
