/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.modeltransformation.dseinternal;

import static org.fortiss.af3.allocation.utils.AllocationModelElementFactory.createAllocationEntry;
import static org.fortiss.af3.exploration.util.DSMLModelElementFactory.createSuperSet;

import java.util.Collection;
import java.util.HashSet;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEntry;
import org.fortiss.af3.exploration.alg.guava.SubClassToInstanceMap;
import org.fortiss.af3.exploration.alg.service.ExplorationTransformationInputs;
import org.fortiss.af3.exploration.alg.service.IExplorationTransformationService.TransformationContext;
import org.fortiss.af3.exploration.dsl_v2.model.expression.SuperSet;
import org.fortiss.af3.partition.model.Partition;
import org.fortiss.af3.partition.model.allocation.PartitionToExecutionUnitAllocationEntry;
import org.fortiss.af3.platform.model.ExecutionUnit;

/**
 * Transforms a {@link PartitionMappingEncoding} to a {@link SuperSet} of AF3
 * {@link PartitionToExecutionUnitAllocationEntry}s.
 * 
 * @author diewald
 */
public class PartitionToExecutionUnitTransformation
		extends SuperSetTransformationBase<PartitionToExecutionUnitAllocationEntry> {
	/** {@inheritDoc} */
	@Override
	public <N extends EObject, S extends EObject> SuperSet<PartitionToExecutionUnitAllocationEntry>
			transform(TransformationState state, TransformationContext context,
					ExplorationTransformationInputs inputs,
					SubClassToInstanceMap<N> transformedModels, EObject outputModel) {
		PartitionMappingEncoding pmEncoding = inputs.getEncoding(PartitionMappingEncoding.class);

		SuperSet<PartitionToExecutionUnitAllocationEntry> allocations =
				createSuperSet("PartToExU", PartitionToExecutionUnitAllocationEntry.class);
		for(PartitionMappingEntry entry : pmEncoding.getMappingEntries()) {
			org.fortiss.af3.partition.model.Partition af3Partition =
					(Partition)encEntryModelElemAssoc.get(entry.getSource(),
							org.fortiss.af3.partition.model.Partition.class);
			assert (af3Partition != null) : "Error in the mapping of internal Partitions --> AF3 Partitions.";
			PartitionToExecutionUnitAllocationEntry af3AllocEntry =
					createAllocationEntry(PartitionToExecutionUnitAllocationEntry.class);
			af3AllocEntry.setSourceElement(af3Partition);
			af3AllocEntry.getTargetElements().add((ExecutionUnit)entry.getTarget().getObject());
			allocations.getEntries().add(af3AllocEntry);
		}
		return allocations;
	}

	/** {@inheritDoc} */
	@Override
	public Collection<Class<?>> getRequiredTypes() {
		Collection<Class<?>> inputTypes = new HashSet<>();
		inputTypes.add(PartitionMappingEncoding.class);
		return inputTypes;
	}

	/** {@inheritDoc} */
	@Override
	public Class<PartitionToExecutionUnitAllocationEntry> getOutputType() {
		return PartitionToExecutionUnitAllocationEntry.class;
	}
}
